@@ -158,6 +158,8 @@ handle_pragma_pack (cpp_reader * ARG_UNUSED (dummy))
     }
   else if (token == CPP_NUMBER)
     {
+      if (TREE_CODE (x) != INTEGER_CST)
+	GCC_BAD ("invalid constant in %<#pragma pack%> - ignored");
       align = TREE_INT_CST_LOW (x);
       action = set;
       if (c_lex (&x) != CPP_CLOSE_PAREN)
@@ -188,6 +190,8 @@ handle_pragma_pack (cpp_reader * ARG_UNUSED (dummy))
 	    }
 	  else if (token == CPP_NUMBER && action == push && align == -1)
 	    {
+	      if (TREE_CODE (x) != INTEGER_CST)
+		GCC_BAD ("invalid constant in %<#pragma pack%> - ignored");
 	      align = TREE_INT_CST_LOW (x);
 	      if (align == -1)
 		action = set;
