@@ -13,25 +13,31 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
+// The ARM simulator does not provide support for "fstat", which
+// causes "in_avail" to return an incorrect value.
+// { dg-do run { xfail arm*-*-elf arm*-*-eabi } }
+
 // 27.6.1.3 unformatted input functions
+// @require@ %-*.tst %-*.txt
+// @diff@ %-*.tst %-*.txt
 
 #include <istream>
-#include <sstream>
+#include <fstream>
 #include <testsuite_hooks.h>
 
 // libstdc++/6746   
-void test12()
+void test13()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
   streamsize sum = 0;
-  wistringstream iss(L"shamma shamma");
+  wifstream ifs("wistream_unformatted-1.tst");
       
   // test01
-  size_t i = iss.rdbuf()->in_avail();
+  size_t i = ifs.rdbuf()->in_avail();
   VERIFY( i != 0 );
     
   // test02
@@ -39,16 +45,16 @@ void test12()
   do
     {
       wchar_t buf[1024];
-      extracted = iss.readsome(buf, sizeof(buf) / sizeof(wchar_t));
+      extracted = ifs.readsome(buf, sizeof(buf) / sizeof(wchar_t));
       sum += extracted;
     }
-  while (iss.good() && extracted);
-  VERIFY( sum != 0 );
+  while (ifs.good() && extracted);
+  VERIFY( sum != 0 );  
 }
-
+ 
 int 
 main()
 {
-  test12();
+  test13();
   return 0;
 }
