@@ -38,7 +38,7 @@ void test06(void)
     iss >> asNum;
     VERIFY( test = iss.eof() );
     VERIFY( test = !iss.fail() );
-    iss.seekg(0, ios_base::beg);
+    iss.seekg(pos1);
     VERIFY( test = !iss.fail() );
   }
 }
