@@ -15,7 +15,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // IA 64 C++ ABI - 5.1 External Names (a.k.a. Mangling)
@@ -27,8 +27,8 @@ int main()
 {
   using namespace __gnu_test;
 
-  // cplus-dem FAIL
-  verify_demangle("_Z1sPA37_iPS0_", "s(int (*) [37], int (**) [37])");
+  verify_demangle("_ZlsRKU3fooU4bart1XS2_", 
+		  "operator<<(X bart foo const&, X bart foo const)");
 
   return 0;
 }
